package main

import (
	"encoding/json"
	"flag"
	"fmt"
	"log"
	"os"
	"strconv"
	"time"

	"scm.dairydemon.net/filifa/mlblive/internal/statsapi"
)

func writeTest(v any, ts string) {
	b, err := json.Marshal(v)
	if err != nil {
		log.Println(err)
	}

	os.WriteFile("examples/data/data_"+ts+".json", b, 0644)
}

func getGamePk() string {
	teamId := flag.String("t", "", "team to get updates for")
	flag.Parse()
	if *teamId == "" {
		log.Fatal("need team ID")
	}

	id, ok := statsapi.TeamIds[*teamId]
	if !ok {
		log.Fatal("invalid team ID")
	}

	sched, err := statsapi.RequestSchedule("1", strconv.Itoa(id))
	if err != nil {
		log.Fatal(err)
	}

	var s statsapi.Schedule
	err = json.Unmarshal(sched, &s)
	if err != nil {
		log.Fatal(err)
	}

	gamePk := s.Dates[0].Games[0].GamePk.String()
	return gamePk
}

func updateFeed(feedResp []byte, gamePk, ts, updateId string) ([]byte, error) {
	diffPatchResp, err := statsapi.RequestDiffPatch(gamePk, ts, updateId)
	if err != nil {
		return statsapi.RequestFeed(gamePk)
	}

	patches, err := diffPatchResp.ExtractPatches()
	if err != nil {
		return statsapi.RequestFeed(gamePk)
	}

	for _, patch := range patches {
		feedResp, err = patch.Apply(feedResp)
		if err != nil {
			break
		}
	}

	return feedResp, err
}

func main() {
	gamePk := getGamePk()

	ws, err := statsapi.NewGamedayWebsocket(gamePk)
	if err != nil {
		log.Fatal(err)
	}
	defer ws.Close()

	ch := make(chan error)
	go ws.KeepAlive(10*time.Second, ch)

	feedResp, err := statsapi.RequestFeed(gamePk)
	if err != nil {
		log.Fatal(err)
	}

	var feed statsapi.Feed
	var game Game
	for {
		err = json.Unmarshal(feedResp, &feed)
		if err != nil {
			log.Fatal(err)
		}
		game.Update(&feed)
		fmt.Println(game)

		ts := feed.MetaData.TimeStamp

		var p statsapi.Push
		err = ws.ReadJSON(&p)
		if err != nil {
			log.Fatal(err)
		}

		feedResp, err = updateFeed(feedResp, gamePk, ts, p.UpdateId)
		if err != nil {
			log.Fatal(err)
		}
	}
}
